/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package moe.nekojimi.nodeprocessor;

import java.util.*;
import moe.nekojimi.nodeprocessor.links.Link;
import moe.nekojimi.nodeprocessor.nodes.Node;

/**
 *
 * @author jimj316
 */
public class Program
{
    private final List<Node> nodes = new ArrayList<>();
    private final Set<Link> links = new HashSet<>();

    private final Queue<Node> executionQueue = new LinkedList<>();
    private final Set<Node> idleNodes = new HashSet<>();

    private Thread thread;
    private State state = State.STOPPED;

    public void addNode(Node node)
    {
        if (!nodes.contains(node))
            nodes.add(node);
        nodes.sort(Node.DEPENDENCY_ORDER);
    }

    public void addLink(Link link)
    {
        links.add(link);
    }

    public void queueNodeProcess(Node node)
    {
        executionQueue.add(node);
    }

    public void setIdleNode(Node node, boolean idle)
    {
        if (idle)
            idleNodes.add(node);
        else
            idleNodes.remove(node);
    }

    public State getState()
    {
        return state;
    }


    public void start()
    {
        if (state != State.STOPPED)
            throw new IllegalStateException("Thread isn't stopped!");
        if (thread == null)
            thread = new Thread(runnable);
        thread.start();
    }

    private final Runnable runnable = () ->
    {
        state = State.RUNNING;
        for (Node node : nodes)
        {
            node.reset();
            if (node.getNumberOfInputs() == 0)
                node.signal();
        }
        while (!executionQueue.isEmpty())
        {
            Node node = executionQueue.poll();
            boolean done = node.process();
            if (!done)
                queueNodeProcess(node);
        }
        state = State.STOPPED;
    };

    public List<Node> getNodes()
    {
        return nodes;
    }

    public Set<Link> getLinks()
    {
        return links;
    }

    public Set<Node> getIdleNodes()
    {
        return idleNodes;
    }

    public enum State
    {
        STOPPED,
        PAUSED,
        RUNNING,
        IDLE
    }
}
