/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package moe.nekojimi.nodeprocessor.ports;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import moe.nekojimi.nodeprocessor.links.Link;
import moe.nekojimi.nodeprocessor.nodes.Node;

/**
 *
 * @author jimj316
 */
public abstract class Port<T>
{
    protected final String name;
    protected final Class<T> clazz;
    protected final Node node;
    protected final List<Link<T, ?, ?>> links = new ArrayList<>();

    public String getName()
    {
        return name;
    }

    public Class<T> getClazz()
    {
        return clazz;
    }

    public void connect(Link<T, ?, ?> link)
    {
        links.add(link);
    }

    public void disconnect(Link<T, ?, ?> link)
    {
        links.remove(link);
    }

    public Port(String name, Class<T> clazz, Node node)
    {
        this.name = name;
        this.clazz = clazz;
        this.node = node;
    }

    public Set<Node> getLinkedNodes()
    {
        Set<Node> ret = new HashSet<>();
        for (Link<T, ?, ?> link : links)
        {
            Port otherSide;
            if (link.getFrom() == this)
                otherSide = link.getTo();
            else
                otherSide = link.getFrom();
            ret.add(otherSide.node);
        }
        return ret;
    }

    public int getNumberOfConnections()
    {
        return links.size();
    }

    public Node getNode()
    {
        return node;
    }

    public List<Link<T, ?, ?>> getLinks()
    {
        return links;
    }

}
